/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.Web;
using System.Web.UI;
using System.Collections;

namespace TopCoder.Web.UI.WebControl.BreadCrumb
{
    /// <summary>
    /// Defines an abstract implementation of <c>IPathWriter</c>. It simplifies the rendering process into four
    /// different tasks: rendering the HTML content before all bread crumb nodes, rendering the HTML content after all
    /// bread crumb nodes, rendering one bread crumb nodes as HTML content, and rendering the HTML content between two
    /// bread crumb nodes.
    /// </summary>
    /// <remarks>
    /// The default implementation does nothing before and after rendering all bread crumb nodes. It renders the path
    /// separator defined by the bread crumb trail control in an HTML span tag. Concrete implementations must provide
    /// the method rendering one bread crumb node. This class would properly process the bread crumb trail according to
    /// the path direction configured in the bread crumb trail control. When rendering one bread crumb node, it supplies
    /// the correct CSS style class name as well. This class is thread-safe, since it is stateless.
    /// </remarks>
    /// <author>Pops</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    public abstract class AbstractPathWriter : IPathWriter
    {
        /// <summary>
        /// Creates a new instance of <c>AbstractPathWriter</c> class.
        /// </summary>
        protected AbstractPathWriter()
        {
        }

        /// <summary>
        /// Processes the given exception according to the error disposition enumeration.
        /// </summary>
        /// <remarks>
        /// When the error disposition is throwing the exception, it returns <c>true</c> indicating the recent exception
        /// should be returned immediately. When the error dispotiion is rendering the exception, it is added to the
        /// current exception list. Otherwise, the exception is ignored.
        /// </remarks>
        /// <param name="error">The exception to be processed.</param>
        /// <param name="errors">The list of existing exceptions.</param>
        /// <param name="errorDisposition">The error disposition indicating how the exception should be processed.
        /// </param>
        /// <returns><c>true</c> if <c>errorDisposition</c> is <c>ThrowException</c>; <c>false</c> otherwise.</returns>
        private bool ProcessException(Exception error, IList errors, ErrorDisposition errorDisposition)
        {
            // If the exception is not a BreadCrumbException, wrap it
            BreadCrumbException exception = (error is BreadCrumbException) ? (BreadCrumbException)error :
                new BreadCrumbException("Error occurs when rendering the bread crumb node.", error);

            switch (errorDisposition)
            {
                case ErrorDisposition.ThrowException:
                    // When the error should be thrown, add the exception to the list and return true
                    errors.Add(exception);

                    return true;

                case ErrorDisposition.RenderException:
                    // When the error should be rendered, add the exception to the list and continue.
                    errors.Add(exception);

                    break;

                case ErrorDisposition.SilentlyIgnore:
                    // When the error should be ignored, ignore the exception
                    break;
            }

            return false;
        }

        /// <summary>
        /// Renders the bread crumb trail to the HTML web page. The given bread crumb node array is from root node to
        /// current node. The node with index 0 is the root node.
        /// </summary>
        /// <remarks>
        /// The bread crumb nodes are rendered according to the path direction configured in the bread crumb trail
        /// control. When the root CSS style is configured in the control, it is used to render the root node. When the
        /// current node CSS style is configured in the control, it is used to render the current node. If any of these
        /// are missing, the normal node CSS style is used if available. If any error occurs during rendering, the error
        /// is processed according to the error disposition flag in the control. When the error is ignored, the process
        /// continues. When the error is rendered, the exception is returned finally while the process continues. When
        /// the error is thrown, the exception is returned immediately with no further process.
        /// </remarks>
        /// <param name="writer">The HTML web page writer where the trail is rendered.</param>
        /// <param name="nodes">The bread crumb trail to be rendered.</param>
        /// <param name="parent">The bread crumb trail control containing the configuration.</param>
        /// <returns><c>null</c> if there is no exception; or exceptions which occur during the rendering.</returns>
        /// <exception cref="ArgumentNullException"><c>writer</c>, <c>nodes</c> or <c>parent</c> is <c>null</c>.
        /// </exception>
        /// <exception cref="ArgumentException"><c>nodes</c> is empty array; or <c>nodes</c> contains <c>null</c> URL.
        /// </exception>
        public virtual BreadCrumbException[] WritePath(HtmlTextWriter writer, BreadCrumbNode[] nodes,
            BreadCrumbTrailControl parent)
        {
            BreadCrumbTrailControlHelper.ValidateNotNull(writer, "writer");
            BreadCrumbTrailControlHelper.ValidateNotNullOrEmpty(nodes, "nodes");
            BreadCrumbTrailControlHelper.ValidateNotNull(parent, "parent");

            ErrorDisposition errorDisposition = parent.ErrorDisposition;
            IList errors = new ArrayList();

            try
            {
                // Initialize the path writer
                BeginPath(writer);
            }
            catch (Exception e)
            {
                if (ProcessException(e, errors, errorDisposition))
                {
                    // When the error should be thrown immediately, it is returned.
                    return new BreadCrumbException[] { (BreadCrumbException)errors[0] };
                }
            }

            int step = 1;
            int first = 0;
            int last = nodes.Length;

            // Which path direction
            switch (parent.PathDirection)
            {
                case PathDirection.RootToCurrent:
                    // When the direction is from root to current
                    // The increment is 1. The first index is 0. The last index is length - 1.
                    step = 1;
                    first = 0;
                    last = nodes.Length - 1;

                    break;

                case PathDirection.CurrentToRoot:
                    // When the direction is from current to root
                    // The increment is -1. The first index is length - 1. The last index is 0.
                    step = -1;
                    first = nodes.Length - 1;
                    last = 0;

                    break;
            }

            // Render all bread crumb nodes
            for (int index = first; index != last + step; index += step)
            {
                string nodeStyle;

                if (index == 0)
                {
                    // When the node is root node, try to use root node style
                    nodeStyle = (parent.RootNodeStyle == null) ? parent.NodeStyle : parent.RootNodeStyle;
                }
                else if (index == nodes.Length - 1)
                {
                    // When the node is current node, try to use current node style
                    nodeStyle = (parent.CurrentNodeStyle == null) ? parent.NodeStyle : parent.CurrentNodeStyle;
                }
                else
                {
                    // Otherwise, use normal node style
                    nodeStyle = parent.NodeStyle;
                }

                try
                {
                    // Render the node
                    WriteNode(writer, nodes[index], nodeStyle, parent.MouseOverStyle);
                }
                catch (Exception e)
                {
                    if (ProcessException(e, errors, errorDisposition))
                    {
                        // When the error should be thrown immediately, it is returned.
                        return new BreadCrumbException[] { (BreadCrumbException)errors[0] };
                    }
                }

                if (index != last)
                {
                    // When the node is not the last node, render the separator
                    try
                    {
                        WriteSeparator(writer, parent);
                    }
                    catch (Exception e)
                    {
                        if (ProcessException(e, errors, errorDisposition))
                        {
                            // When the error should be thrown immediately, it is returned.
                            return new BreadCrumbException[] { (BreadCrumbException)errors[0] };
                        }
                    }
                }
            }

            try
            {
                // End the path writer
                EndPath(writer);
            }
            catch (Exception e)
            {
                if (ProcessException(e, errors, errorDisposition))
                {
                    // When the error should be thrown immediately, it is returned.
                    return new BreadCrumbException[] { (BreadCrumbException)errors[0] };
                }
            }

            BreadCrumbException[] exceptions = null;

            if (errors.Count != 0)
            {
                // When there is any error, copy the errors to the array
                exceptions = new BreadCrumbException[errors.Count];
                errors.CopyTo(exceptions, 0);
            }

            return exceptions;
        }

        /// <summary>
        /// Renders the HTML content before all bread crumb nodes.
        /// </summary>
        /// <remarks>
        /// The default implementation does nothing except argument validation.
        /// </remarks>
        /// <param name="writer">The HTML web page writer where the content is rendered.</param>
        /// <exception cref="ArgumentNullException"><c>writer</c> is <c>null</c>.</exception>
        /// <exception cref="BreadCrumbException">Rendering the HTML content fails.</exception>
        protected virtual void BeginPath(HtmlTextWriter writer)
        {
            BreadCrumbTrailControlHelper.ValidateNotNull(writer, "writer");
        }

        /// <summary>
        /// Renders a single bread crumb node in the HTML web page. The style of the node and the style when mouse
        /// hovers over the content are given.
        /// </summary>
        /// <remarks>
        /// When the node style or the mouse over style is <c>null</c>, the node style and/or the mouse over style is
        /// not used. The title of the bread crumb node should be rendered as the text, and the URl of the bread crumb
        /// node should be used as the hyperlink of the rendered content.
        /// </remarks>
        /// <param name="writer">The HTML web page writer where the bread crumb node is rendered.</param>
        /// <param name="node">The bread crumb node to be rendered.</param>
        /// <param name="nodeStyle">The node CSS style class used to render the node.</param>
        /// <param name="mouseOverStyle">The CSS style class when the mouse hovers over the rendered node.</param>
        /// <exception cref="ArgumentNullException"><c>writer</c> or <c>node</c> is <c>null</c>.</exception>
        /// <exception cref="BreadCrumbException">Rendering the bread crumb node fails.</exception>
        protected abstract void WriteNode(HtmlTextWriter writer, BreadCrumbNode node, string nodeStyle,
            string mouseOverStyle);

        /// <summary>
        /// Renders the separator between two bread crumb nodes.
        /// </summary>
        /// <remarks>
        /// The default implementation renders the path separator configured in the bread crumb trail control. If the
        /// path separator is <c>null</c>, nothing is rendered.
        /// </remarks>
        /// <param name="writer">The HTML web page writer where the path separator is rendered.</param>
        /// <param name="parent">The bread crumb trail control containing the configuration.</param>
        /// <exception cref="ArgumentNullException"><c>writer</c> or <c>parent</c> is <c>null</c>.</exception>
        /// <exception cref="BreadCrumbException">Rendering the path separator fails.</exception>
        protected virtual void WriteSeparator(HtmlTextWriter writer, BreadCrumbTrailControl parent)
        {
            BreadCrumbTrailControlHelper.ValidateNotNull(writer, "writer");
            BreadCrumbTrailControlHelper.ValidateNotNull(parent, "parent");

            string separator = parent.PathSeparator;

            if (separator != null)
            {
                // When the path separator is not null, output it.
                string style = parent.PathSeparatorStyle;

                if (style != null)
                {
                    // When the CSS style is not null, write it
                    writer.AddAttribute(HtmlTextWriterAttribute.Class, style);
                }

                try
                {
                    // Render the SPAN tag and the separator
                    writer.RenderBeginTag(HtmlTextWriterTag.Span);
                    writer.Write(HttpUtility.HtmlEncode(separator));
                    writer.RenderEndTag();
                }
                catch (Exception e)
                {
                    throw new BreadCrumbException("Rendering the bread crumb node separator fails.", e);
                }
            }
        }

        /// <summary>
        /// Renders the HTML content after all bread crumb nodes.
        /// </summary>
        /// <remarks>
        /// The default implementation does nothing except argument validation.
        /// </remarks>
        /// <param name="writer">The HTML web page writer where the content is rendered.</param>
        /// <exception cref="ArgumentNullException"><c>writer</c> is <c>null</c>.</exception>
        /// <exception cref="BreadCrumbException">Rendering the HTML content fails.</exception>
        protected virtual void EndPath(HtmlTextWriter writer)
        {
            BreadCrumbTrailControlHelper.ValidateNotNull(writer, "writer");
        }
    }
}
