/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;

namespace TopCoder.Web.UI.WebControl.BreadCrumb
{
    /// <summary>
    /// Defines a bread crumb node representing a page context. It contains a pair of the title and the URL of the page
    /// context. Generally, the title represents the content to be rendered if this node is on the bread crumb trail.
    /// The URL represents the hyperlink where the page context need to be re-visited. The title information may be
    /// <c>null</c>. In this case, the title in the site map should be used.
    /// </summary>
    /// <remarks>
    /// This class is thread-safe, since it is immutable.
    /// </remarks>
    /// <author>Pops</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [Serializable]
    public struct BreadCrumbNode
    {
        /// <summary>
        /// Represents the title of the page context represented by this node.
        /// </summary>
        private readonly string title;

        /// <summary>
        /// Represents the URL of the page context represented by this node.
        /// </summary>
        private readonly string url;

        /// <summary>
        /// Gets the title of the page context represented by this node.
        /// </summary>
        /// <value>The title of the page context.</value>
        public string Title
        {
            get
            {
                return title;
            }
        }

        /// <summary>
        /// Gets the URL of the page context represented by this node.
        /// </summary>
        /// <value>The URL of the page context.</value>
        public string Url
        {
            get
            {
                return url;
            }
        }

        /// <summary>
        /// Creates a new instance of <c>BreadCrumbNode</c> structure. The title and the URL of the page context
        /// represented by this bread crumb node are given. When the title is empty string, it is set as <c>null</c>.
        /// </summary>
        /// <param name="title">The title of the page context to be represented.</param>
        /// <param name="url">The URL of the page context to be represented.</param>
        /// <exception cref="ArgumentNullException"><c>url</c> is <c>null</c>.</exception>
        /// <exception cref="ArgumentException"><c>url</c> is empty string.</exception>
        public BreadCrumbNode(string title, string url)
        {
            BreadCrumbTrailControlHelper.ValidateNotNullOrEmpty(url, "url");

            this.title = ((title == null) || (title.Trim().Length == 0)) ? null : title;
            this.url = url;
        }
    }
}

