/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.Collections;
using TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery;

namespace TopCoder.Web.UI.WebControl.BreadCrumb
{
    /// <summary>
    /// Defines helper methods used in this component.
    /// </summary>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    internal sealed class BreadCrumbTrailControlHelper
    {
        /// <summary>
        /// Creates a new instance of <c>BreadCrumbTrailControlHelper</c> class.
        /// </summary>
        /// <remarks>
        /// This private constructor prevents the creation of a new instance.
        /// </remarks>
        private BreadCrumbTrailControlHelper()
        {
        }

        /// <summary>
        /// Validates the value of a variable. The value cannot be <c>null</c>.
        /// </summary>
        /// <param name="value">The value of the variable to be validated.</param>
        /// <param name="name">The name of the variable to be validated.</param>
        /// <exception cref="ArgumentNullException">The value of the variable is <c>null</c>.</exception>
        public static void ValidateNotNull(object value, string name)
        {
            if (value == null)
            {
                throw new ArgumentNullException(name, name + " cannot be null.");
            }
        }

        /// <summary>
        /// Validates the value of a string variable. The value cannot be <c>null</c> or empty string after trimming.
        /// </summary>
        /// <param name="value">The value of the variable to be validated.</param>
        /// <param name="name">The name of the variable to be validated.</param>
        /// <exception cref="ArgumentNullException">The value of the variable is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">The value of the variable is empty string.</exception>
        public static void ValidateNotNullOrEmpty(string value, string name)
        {
            ValidateNotNull(value, name);

            if (value.Trim().Length == 0)
            {
                throw new ArgumentException(name + " cannot be empty string.", name);
            }
        }

        /// <summary>
        /// Validates the value of an array bread crumb node variable. The value cannot be <c>null</c> or empty array.
        /// The URLs of the bread crumb nodes should not be <c>null</c>
        /// </summary>
        /// <param name="value">The value of the variable to be validated.</param>
        /// <param name="name">The name of the variable to be validated.</param>
        /// <exception cref="ArgumentNullException">The value of the variable is <c>null</c>.</exception>
        /// <exception cref="ArgumentException">The value of the variable is empty array; or the URL of the node is
        /// <c>null</c>.</exception>
        public static void ValidateNotNullOrEmpty(BreadCrumbNode[] value, string name)
        {
            ValidateNotNull(value, name);

            if (value.Length == 0)
            {
                throw new ArgumentException(name + " cannot be empty array.", name);
            }

            foreach (BreadCrumbNode node in value)
            {
                if (node.Url == null)
                {
                    throw new ArgumentException(name + " cannot contain null URLs.", name);
                }
            }
        }

        /// <summary>
        /// Validates the value of an enumeration variable. The value cannot be undefined by the type.
        /// </summary>
        /// <param name="value">The value of the variable to be validated.</param>
        /// <param name="name">The name of the variable to be validated.</param>
        /// <param name="type">The type of the variable to be validated.</param>
        /// <exception cref="ArgumentException">The value of the variable is not defined by the type.</exception>
        public static void ValidateEnumeration(Enum value, string name, Type type)
        {
            if (!Enum.IsDefined(type, value))
            {
                throw new ArgumentException(name + " is an invalid enumeration value.", name);
            }
        }

        /// <summary>
        /// Converts a collection into a set. The keys of the returned dictionary represents the set. The values are
        /// <c>null</c>.
        /// </summary>
        /// <param name="collection">The collection which is converted into a set.</param>
        /// <returns>A dictionary whose keys form the set created from the collection.</returns>
        private static IDictionary ToSet(ICollection collection)
        {
            IDictionary dict = new Hashtable(collection.Count);

            foreach (object obj in collection)
            {
                dict.Add(obj, null);
            }

            return dict;
        }

        /// <summary>
        /// Gets a reversed site map from the given site map. The keys of the reversed site map represents the child,
        /// while the values of the reversed site map represents the list of parents. When the key is a root node, the
        /// value is an empty list.
        /// </summary>
        /// <param name="siteMap">The site map from which the reversed site map is obtained.</param>
        /// <returns>The reversed site map obtained from the given site map.</returns>
        public static IDictionary GetReverseSiteMap(IDictionary siteMap)
        {
            // Copy all node matchers from the site map.
            IDictionary reverseSiteMap = new Hashtable();

            foreach (object obj in siteMap.Keys)
            {
                reverseSiteMap.Add(obj, new ArrayList());
            }

            // Enumerate every parent in the site map
            foreach (DictionaryEntry entry in siteMap)
            {
                INodeMatcher parent = (INodeMatcher)entry.Key;
                IList children = (IList)entry.Value;

                // Enumerate every child under the parent
                foreach (INodeMatcher child in children)
                {
                    IList parentList = (IList)reverseSiteMap[child];

                    if (parentList == null)
                    {
                        // When the child is not in the parent list, create the entry
                        parentList = new ArrayList();
                        reverseSiteMap[child] = parentList;
                    }

                    // Add this parent to the parent list of the child
                    parentList.Add(parent);
                }
            }

            return reverseSiteMap;
        }

        /// <summary>
        /// Searches the bread crumb trail from the current node to one of the root nodes.
        /// </summary>
        /// <remarks>
        /// The searching is done based on the previous bread crumb trail and the structure of the web site. It chooses
        /// the shortest path which has the most common nodes shared with the previous path starting from the root.
        /// </remarks>
        /// <param name="currentNode">The current bread crumb node whose trail is searched.</param>
        /// <param name="previousPath">The previous bread crumb trail.</param>
        /// <param name="reverseSiteMap">The reversed site map used to search the trail.</param>
        /// <param name="path">The searched bread crumb trail.</param>
        /// <exception cref="BreadCrumbException">The reverse site map is invalid.</exception>
        public static void SearchPath(BreadCrumbNode currentNode, NodePair[] previousPath, IDictionary reverseSiteMap,
            IList path)
        {
            // Set the current known best length and the best common nodes shared with the previous path
            int bestLength = int.MaxValue;
            int bestCommon = int.MinValue;
            IList roots = new ArrayList();

            // Enumerate all node matchers
            try
            {
                // Get all root nodes
                foreach (DictionaryEntry entry in reverseSiteMap)
                {
                    if (((IList)entry.Value).Count == 0)
                    {
                        // The parent list of the root node is null
                        roots.Add(entry.Key);
                    }
                }

                foreach (INodeMatcher matcher in reverseSiteMap.Keys)
                {
                    if (matcher.Matches(currentNode))
                    {
                        // When the current node matches the matcher, search the shortest path using this as start
                        // Use Dijkstra algorithm to calculate the shortest path
                        IDictionary distance = new Hashtable();
                        IDictionary pathFrom = new Hashtable();
                        Queue queue = new Queue();

                        distance[matcher] = 0;
                        queue.Enqueue(matcher);

                        while (queue.Count > 0)
                        {
                            // Get the first matcher
                            INodeMatcher currentMatcher = (INodeMatcher)queue.Dequeue();
                            int currentDistance = (int)distance[currentMatcher];

                            foreach (INodeMatcher next in (IList)reverseSiteMap[currentMatcher])
                            {
                                if (!distance.Contains(next))
                                {
                                    // When the distance from the start matcher to the matcher is not available, the
                                    // current distance + 1 is the shortest distance.
                                    pathFrom[next] = currentMatcher;
                                    distance[next] = currentDistance + 1;

                                    // Enqueue the matcher
                                    queue.Enqueue(next);
                                }
                            }
                        }

                        // Here, the shortest path to all possible nodes are calculated.
                        // It is safe to assume the previous trail is a valid one.
                        // In this case, we must find a most common node sharing shortest path.
                        // The path length should be the length from the previous root to the index node, plus the
                        // length of the shortest path from the index node to the start node.
                        for (int previousPathIndex = previousPath.Length - 1; previousPathIndex >= 0;
                            --previousPathIndex)
                        {
                            INodeMatcher indexMatcher = previousPath[previousPathIndex].NodeMatcher;
                            int pathLength = distance.Contains(indexMatcher) ?
                                (previousPathIndex + (int)distance[indexMatcher]) : -1;

                            if ((pathLength != -1) && ((pathLength < bestLength) || ((pathLength == bestLength) &&
                                (previousPathIndex + 1 > bestCommon))))
                            {
                                // When the path length is shorter than the best known one,
                                // or the path length equals to the best known one, but it has more common nodes,
                                // the current path should be used.
                                bestLength = pathLength;
                                bestCommon = previousPathIndex + 1;

                                path.Clear();

                                // Add the common nodes into the path
                                for (int index = 0; index <= previousPathIndex; ++index)
                                {
                                    path.Add(previousPath[index]);
                                }

                                // Add the shortest path from the index node to the start node
                                // The index node is excluded, since it is already in the path
                                BuildPath(pathFrom, (INodeMatcher)pathFrom[indexMatcher], currentNode, path);

                                // Still need to continue, since less common nodes may result shorter path
                            }
                        }

                        // Enumerate all roots, without considering the common nodes
                        foreach (INodeMatcher rootMatcher in roots)
                        {
                            if (distance.Contains(rootMatcher) && (int)distance[rootMatcher] < bestLength)
                            {
                                // When the path is shorter than the best known, though it is considered as no common
                                // nodes, it should still be used.
                                bestLength = (int)distance[rootMatcher];
                                bestCommon = 0;

                                path.Clear();

                                // Build the shortest path
                                BuildPath(pathFrom, rootMatcher, currentNode, path);
                            }
                        }
                    }
                }
            }
            catch (InvalidCastException e)
            {
                throw new BreadCrumbException("The reverse site map is invalid.", e);
            }
        }

        /// <summary>
        /// Builds the shortest path starting from the given node matcher. The dictionary storing the previous node
        /// matcher in the shortest path is given. The path should be appended to the given path list. The last node in
        /// the path uses the given bread crumb node information.
        /// </summary>
        /// <param name="pathFrom">The dictionary storing the previous node matcher in the shortest path.</param>
        /// <param name="start">The start node of the shortest path to be built.</param>
        /// <param name="endNode">The bread crumb node for the last node in the shortest path.</param>
        /// <param name="path">The path list where the shortest path is appended.</param>
        private static void BuildPath(IDictionary pathFrom, INodeMatcher start, BreadCrumbNode endNode, IList path)
        {
            INodeMatcher current = start;

            while (current != null)
            {
                INodeMatcher next = (INodeMatcher)pathFrom[current];

                // When the next node is null, it is last node in the path, so the given bread crumb node is used.
                // Otherwise, a bread crumb node is created using the matcher's title and URL.
                path.Add(new NodePair((next == null) ? endNode : new BreadCrumbNode(current.Title, current.Url),
                    current));

                current = next;
            }
        }

        /// <summary>
        /// Gets the bread crumb trail as the array of bread crumb nodes. The original trail is represented by a list
        /// of <c>NodePair</c> instances.
        /// </summary>
        /// <param name="path">The original bread crumb trail represented by <c>NodePair</c> instances.</param>
        /// <returns>An array of bread crumb nodes extracted from the <c>NodePair</c> instances.</returns>
        public static BreadCrumbNode[] GetPathNodes(IList path)
        {
            if (path.Count == 0)
            {
                return null;
            }

            BreadCrumbNode[] nodes = new BreadCrumbNode[path.Count];

            for (int index = 0; index < nodes.Length; ++index)
            {
                nodes[index] = ((NodePair)path[index]).Node;
            }

            return nodes;
        }
    }
}

