/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.Web;
using System.Text;
using System.Web.UI;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.NodeDiscovery
{
    /// <summary>
    /// Implements <c>INodeDiscovery</c> interface to provide the functionality to obtain the bread crumb node according
    /// to the HTTP request of the page context.
    /// </summary>
    /// <remarks>
    /// This class is thread-safe, since it is stateless.
    /// </remarks>
    /// <author>Pops</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    public class HttpRequestNodeDiscovery : INodeDiscovery
    {
        /// <summary>
        /// Creates a new instance of <c>HttpRequestNodeDiscovery</c> class.
        /// </summary>
        public HttpRequestNodeDiscovery()
        {
        }

        /// <summary>
        /// Gets the bread crumb node representing the given page context. The title of the bread crumb node may be
        /// given.
        /// </summary>
        /// <remarks>
        /// The title of the returned node should be the given title. The URL of the page context is extracted from the
        /// HTTP request of the page. If there is any query string, it is rebuilt and appended to the URL properly. The
        /// query string is rebuild in the ascending order of the query key name.
        /// </remarks>
        /// <param name="title">The title of the page.</param>
        /// <param name="page">The page context used to create the bread crumb node.</param>
        /// <returns>The bread crumb node representation of the given page context using the given title.</returns>
        /// <exception cref="ArgumentNullException"><c>page</c> is <c>null</c>.</exception>
        /// <exception cref="BreadCrumbException">The HTTP request information is not available in the page context;
        /// or the URL is empty string.</exception>
        public BreadCrumbNode GetCurrentNode(string title, Page page)
        {
            BreadCrumbTrailControlHelper.ValidateNotNull(page, "page");

            try
            {
                StringBuilder url = new StringBuilder();
                HttpRequest request = page.Request;
                string[] keys = request.QueryString.AllKeys;

                url.Append(request.Path);

                if (keys.Length != 0)
                {
                    // When there is query string after the URL, reconstruct it.
                    url.Append('?');

                    // Sort all query string keys
                    Array.Sort(keys);

                    bool first = true;

                    // Recreate the query string
                    foreach (string key in keys)
                    {
                        // The first name/value pair does not need '&'
                        if (first)
                        {
                            first = false;
                        }
                        else
                        {
                            url.Append('&');
                        }

                        url.Append(key);
                        url.Append('=');
                        url.Append(request.QueryString[key]);
                    }
                }

                // Create the node with the title and the url
                return new BreadCrumbNode(title, url.ToString());
            }
            catch (ArgumentException e)
            {
                throw new BreadCrumbException("The URL is empty string.", e);
            }
            catch (HttpException e)
            {
                throw new BreadCrumbException("HTTP error occurs.", e);
            }
        }
    }
}

