/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.Web.UI;
using System.Collections;
using NUnit.Framework;
using NUnit.Extensions.Asp;
using NUnit.Extensions.Asp.AspTester;
using TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.NodeMatcher;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery
{
    /// <summary>
    /// Tests the functionality and error cases of <c>DynamicPathDiscovery</c> class. Some test cases are in the
    /// ASP.NET page, and accessed via NUnitAsp.
    /// </summary>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TestFixture]
    public class DynamicPathDiscoveryTestCase : WebFormTestCase
    {
        /// <summary>
        /// Represents the <c>DynamicPathDiscovery</c> instance used in tests.
        /// </summary>
        private IPathDiscovery pathDiscovery;

        /// <summary>
        /// Sets up the test environment. The test instance is created.
        /// </summary>
        protected override void SetUp()
        {
            pathDiscovery = new DynamicPathDiscovery();
        }

        /// <summary>
        /// Cleans up the test environment. The test instance is disposed.
        /// </summary>
        protected override void TearDown()
        {
            pathDiscovery = null;
        }

        /// <summary>
        /// Tests <c>GetPath(BreadCrumbNode, Page)</c> when the given page is <c>null</c>. Expect ArgumentNullException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestGetPathBreadCrumbNodePagePageNull()
        {
            pathDiscovery.GetPath(new BreadCrumbNode("title", "node"), null);
        }

        /// <summary>
        /// Tests <c>GetPath(BreadCrumbNode, Page)</c> when the session of the given page is not available. Expect
        /// BreadCrumbException.
        /// </summary>
        [Test, ExpectedException(typeof(BreadCrumbException))]
        public void TestGetPathBreadCrumbNodePageNoSession()
        {
            pathDiscovery.GetPath(new BreadCrumbNode("title", "node"), new Page());
        }

        /// <summary>
        /// Tests <c>GetPath(BreadCrumbNode, Page)</c> when the URL of the given node is <c>null</c>. Expect
        /// ArgumentNullException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestGetPathBreadCrumbNodePageNoURL()
        {
            pathDiscovery.GetPath(new BreadCrumbNode(), new Page());
        }

        /// <summary>
        /// Tests <c>GetPath(BreadCrumbNode, Page)</c> when retrieveing the site map fails. Expect BreadCrumbException.
        /// This is verified by the ASP.NET test page via NUnitAsp.
        /// </summary>
        [Test]
        public void TestGetPathBreadCrumbNodePageGetSiteMapFail()
        {
            Browser.GetPage("http://localhost/test/DynamicPathDiscoveryTest.aspx");

            ButtonTester button = new ButtonTester("GetPathBreadCrumbNodePageGetSiteMapFail", CurrentWebForm);

            button.Click();
        }

        /// <summary>
        /// Tests <c>GetPath(BreadCrumbNode, Page)</c> when the reversed site map is invalid. Expect
        /// BreadCrumbException. This is verified by the ASP.NET test page via NUnitAsp.
        /// </summary>
        [Test]
        public void TestGetPathBreadCrumbNodePageSiteMapInvalid()
        {
            Browser.GetPage("http://localhost/test/DynamicPathDiscoveryTest.aspx");

            ButtonTester button = new ButtonTester("GetPathBreadCrumbNodePageSiteMapInvalid", CurrentWebForm);

            button.Click();
        }

        /// <summary>
        /// Tests <c>GetPath(BreadCrumbNode, Page)</c> when creating the ISiteDiscovery instance fails. Expect
        /// BreadCrumbException. This is verified by the ASP.NET test page via NUnitAsp.
        /// </summary>
        [Test]
        public void TestGetPathBreadCrumbNodePageISiteDiscoveryCreationFail()
        {
            Browser.GetPage("http://localhost/test/DynamicPathDiscoveryTest.aspx");

            ButtonTester button = new ButtonTester("GetPathBreadCrumbNodePageISiteDiscoveryCreationFail",
                CurrentWebForm);

            button.Click();
        }

        /// <summary>
        /// Tests <c>GetPath(BreadCrumbNode, Page)</c> when the ISiteDiscovery instance cannot be found in the object
        /// factory. Expect BreadCrumbException. This is verified by the ASP.NET test page via NUnitAsp.
        /// </summary>
        [Test]
        public void TestGetPathBreadCrumbNodePageISiteDiscoverySourceFail()
        {
            Browser.GetPage("http://localhost/test/DynamicPathDiscoveryTest.aspx");

            ButtonTester button = new ButtonTester("GetPathBreadCrumbNodePageISiteDiscoverySourceFail",
                CurrentWebForm);

            button.Click();
        }

        /// <summary>
        /// Tests <c>GetPath(BreadCrumbNode, Page)</c> when the ISiteDiscovery instance is not an ISiteDiscovery
        /// instance. Expect BreadCrumbException. This is verified by the ASP.NET test page via NUnitAsp.
        /// </summary>
        [Test]
        public void TestGetPathBreadCrumbNodePageISiteDiscoveryWrongType()
        {
            Browser.GetPage("http://localhost/test/DynamicPathDiscoveryTest.aspx");

            ButtonTester button = new ButtonTester("GetPathBreadCrumbNodePageISiteDiscoveryWrongType",
                CurrentWebForm);

            button.Click();
        }

        /// <summary>
        /// Tests <c>GetPath(BreadCrumbNode, Page)</c> when the configuration for the object factory is invalid. Expect
        /// BreadCrumbException. This is verified by the ASP.NET test page via NUnitAsp.
        /// </summary>
        [Test]
        public void TestGetPathBreadCrumbNodePageInvalidConfig()
        {
            Browser.GetPage("http://localhost/test/DynamicPathDiscoveryTest.aspx");

            ButtonTester button = new ButtonTester("GetPathBreadCrumbNodePageInvalidConfig", CurrentWebForm);

            button.Click();
        }

        /// <summary>
        /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when two matchers matches the given bread crumb node.
        /// There is no previous path available. The shortest path should be selected. Since the title of the node is
        /// <c>null</c>, the title in the matcher is used. This is verified by the ASP.NET test page via NUnitAsp.
        /// </summary>
        [Test]
        public void TestGetPathBreadCrumbNodePageTwoMatchedAccuracy()
        {
            Browser.GetPage("http://localhost/test/DynamicPathDiscoveryTest.aspx");

            ButtonTester button = new ButtonTester("GetPathBreadCrumbNodePageTwoMatchedAccuracy", CurrentWebForm);

            button.Click();
        }

        /// <summary>
        /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when one matched matcher has multiple roots. The
        /// shortest path should be selected. The title of the given bread crumb node should be used. This is verified
        /// by the ASP.NET test page via NUnitAsp.
        /// </summary>
        [Test]
        public void TestGetPathBreadCrumbNodePageMultipleRootsAccuracy()
        {
            Browser.GetPage("http://localhost/test/DynamicPathDiscoveryTest.aspx");

            ButtonTester button = new ButtonTester("GetPathBreadCrumbNodePageMultipleRootsAccuracy", CurrentWebForm);

            button.Click();
        }

        /// <summary>
        /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when no matched matcher has been found. There is a
        /// previous path available.  It should return the previous path appending this node. This is verified by the
        /// ASP.NET test page via NUnitAsp.
        /// </summary>
        [Test]
        public void TestGetPathBreadCrumbNodePageNoMatchAccuracy()
        {
            Browser.GetPage("http://localhost/test/DynamicPathDiscoveryTest.aspx");

            ButtonTester button = new ButtonTester("GetPathBreadCrumbNodePageNoMatchAccuracy", CurrentWebForm);

            button.Click();
        }

        /// <summary>
        /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when looped matcher matches the node. There is no
        /// previous path available. It should return <c>null</c>, since there is no path to the root. This is verified
        /// by the ASP.NET test page via NUnitAsp.
        /// </summary>
        [Test]
        public void TestGetPathBreadCrumbNodePageLoopAccuracy()
        {
            Browser.GetPage("http://localhost/test/DynamicPathDiscoveryTest.aspx");

            ButtonTester button = new ButtonTester("GetPathBreadCrumbNodePageLoopAccuracy", CurrentWebForm);

            button.Click();
        }

        /// <summary>
        /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when one normal matcher matches the node. There is no
        /// previous path available. It should return the shortest path. This is verified by the ASP.NET test page via
        /// NUnitAsp.
        /// </summary>
        [Test]
        public void TestGetPathBreadCrumbNodePageNormalAccuracy()
        {
            Browser.GetPage("http://localhost/test/DynamicPathDiscoveryTest.aspx");

            ButtonTester button = new ButtonTester("GetPathBreadCrumbNodePageNormalAccuracy", CurrentWebForm);

            button.Click();
        }

        /// <summary>
        /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when the current node is on the previous path. The
        /// previous path should be used directly. The URLs of the previous path and the current node should be
        /// preserved. This is verified by the ASP.NET test page via NUnitAsp.
        /// </summary>
        [Test]
        public void TestGetPathBreadCrumbNodePageOnPreviousPathAccuracy()
        {
            Browser.GetPage("http://localhost/test/DynamicPathDiscoveryTest.aspx");

            ButtonTester button = new ButtonTester("GetPathBreadCrumbNodePageOnPreviousPathAccuracy", CurrentWebForm);

            button.Click();
        }

        /// <summary>
        /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when the current node is the child of the last node
        /// in the previous path. The previous path should be used directly appending the current node. The URLs of the
        /// previous path and the current node should be preserved. This is verified by the ASP.NET test page via
        /// NUnitAsp.
        /// </summary>
        [Test]
        public void TestGetPathBreadCrumbNodePageAppendPreviousPathAccuracy()
        {
            Browser.GetPage("http://localhost/test/DynamicPathDiscoveryTest.aspx");

            ButtonTester button = new ButtonTester("GetPathBreadCrumbNodePageAppendPreviousPathAccuracy",
                CurrentWebForm);

            button.Click();
        }

        /// <summary>
        /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when the current node is an unknown node. The
        /// previous path should be used directly appending the current node. The URLs of the previous path and the
        /// current node should be preserved. This is verified by the ASP.NET test page via NUnitAsp.
        /// </summary>
        [Test]
        public void TestGetPathBreadCrumbNodePagePreviousPathUnknownAccuracy()
        {
            Browser.GetPage("http://localhost/test/DynamicPathDiscoveryTest.aspx");

            ButtonTester button = new ButtonTester("GetPathBreadCrumbNodePagePreviousPathUnknownAccuracy",
                CurrentWebForm);

            button.Click();
        }

        /// <summary>
        /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when the shortest path shares no node with the
        /// previous path while there is a longer path which shares some nodes with the previous path. The shortest no
        /// common path should be chosen. This is verified by the ASP.NET test page via NUnitAsp.
        /// </summary>
        [Test]
        public void TestGetPathBreadCrumbNodePageShortestNoCommonAccuracy()
        {
            Browser.GetPage("http://localhost/test/DynamicPathDiscoveryTest.aspx");

            ButtonTester button = new ButtonTester("GetPathBreadCrumbNodePageShortestNoCommonAccuracy",
                CurrentWebForm);

            button.Click();
        }

        /// <summary>
        /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when the shortest path shares less node with the
        /// previous path while there is a longer path which shares more nodes with the previous path. The shortest less
        /// common path should be chosen. This is verified by the ASP.NET test page via NUnitAsp.
        /// </summary>
        [Test]
        public void TestGetPathBreadCrumbNodePageShortestLessCommonAccuracy()
        {
            Browser.GetPage("http://localhost/test/DynamicPathDiscoveryTest.aspx");

            ButtonTester button = new ButtonTester("GetPathBreadCrumbNodePageShortestLessCommonAccuracy",
                CurrentWebForm);

            button.Click();
        }

        /// <summary>
        /// Tests accuracy of <c>GetPath(BreadCrumbNode, Page)</c> when the two shortest paths exists. The path which
        /// shares more common nodes with the previous path should be chosen. This is verified by the ASP.NET test page
        /// via NUnitAsp.
        /// </summary>
        [Test]
        public void TestGetPathBreadCrumbNodePageMoreCommonAccuracy()
        {
            Browser.GetPage("http://localhost/test/DynamicPathDiscoveryTest.aspx");

            ButtonTester button = new ButtonTester("GetPathBreadCrumbNodePageMoreCommonAccuracy",
                CurrentWebForm);

            button.Click();
        }
    }
}

