/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using NUnit.Framework;
using TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.NodeMatcher;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery
{
    /// <summary>
    /// Tests the functionality and error cases of <c>NodePair</c> class. Since this is an immutable class, the property
    /// getters are tested together with the constructor.
    /// </summary>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TestFixture]
    public class NodePairTestCase
    {
        /// <summary>
        /// Represents a valid <c>INodeMatcher</c> instance used in tests.
        /// </summary>
        private static readonly INodeMatcher MATCHER = new RegexNodeMatcher("title", "url", ".*");

        /// <summary>
        /// Tests <c>NodePair(BreadCrumbNode, INodeMatcher)</c> when the given node matcher is <c>null</c>. Expect
        /// ArgumentNullException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestNodePairBreadCrumbNodeINodeMatcherMatcherNull()
        {
            new NodePair(new BreadCrumbNode("title", "url"), null);
        }

        /// <summary>
        /// Tests <c>NodePair(BreadCrumbNode, INodeMatcher)</c> when the URL of the given node is <c>null</c>. Expect
        /// ArgumentNullException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestNodePairBreadCrumbNodeINodeMatcherNodeURLNull()
        {
            new NodePair(new BreadCrumbNode(), MATCHER);
        }

        /// <summary>
        /// Tests accuracy of <c>NodePair(BreadCrumbNode, INodeMatcher)</c> when the title of the bread crumb node is
        /// <c>null</c>. The node matcher should be retrieved as given. The bread crumb node should be a node with the
        /// title of the node matcher and the URL of the given bread crumb node. This is also a test case of property
        /// getters.
        /// </summary>
        [Test]
        public void TestNodePairBreadCrumbNodeINodeMatcherNodeTitleNullAccuracy()
        {
            NodePair pair = new NodePair(new BreadCrumbNode(null, "node url"), MATCHER);

            // Verify
            Assert.AreSame(MATCHER, pair.NodeMatcher, "The bread crumb node matcher should be the one as given.");
            Assert.AreEqual(MATCHER.Title, pair.Node.Title,
                "The bread crumb node title should be the one in the matcher.");
            Assert.AreEqual("node url", pair.Node.Url, "The URL should be the original URL of the bread crumb node.");
        }

        /// <summary>
        /// Tests accuracy of <c>NodePair(BreadCrumbNode, INodeMatcher)</c> when the title of the bread crumb node is
        /// not <c>null</c>. The node matcher should be retrieved as given. The bread crumb node should be the original
        /// bread crumb node as given. These are verified by the property getters. This is also a test case of property
        /// getters.
        /// </summary>
        [Test]
        public void TestNodePairBreadCrumbNodeINodeMatcherMatcherNormalAccuracy()
        {
            BreadCrumbNode node = new BreadCrumbNode("node title", "node url");
            NodePair pair = new NodePair(node, MATCHER);

            // Verify
            Assert.AreSame(MATCHER, pair.NodeMatcher, "The bread crumb node matcher should be the one as given.");
            Assert.AreEqual(node, pair.Node, "The bread crumb node should be the one as given.");
        }
    }
}

