/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.Reflection;
using System.Collections;
using System.Security.Permissions;
using NUnit.Framework;
using TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery;

namespace TopCoder.Web.UI.WebControl.BreadCrumb
{
    /// <summary>
    /// Defines utilities used in test cases.
    /// </summary>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    public sealed class TestHelper
    {
        /// <summary>
        /// Creates a new instance of <c>TestHelper</c> class.
        /// </summary>
        /// <remarks>
        /// This private constructor prevents the creation of a new instance.
        /// </remarks>
        private TestHelper()
        {
        }

        /// <summary>
        /// Invokes a non-public method defined in the given instance. The method name and the parameters are given.
        /// When there is an invocation exception, the original exception is thrown.
        /// </summary>
        /// <param name="instance">The instance whose method is invoked.</param>
        /// <param name="methodName">The name of the method to be invoked.</param>
        /// <param name="param">The parameters used to invoke the method.</param>
        /// <returns>The return value of the invoked method if any.</returns>
        [ReflectionPermission(SecurityAction.Demand, MemberAccess=true)]
        public static object Invoke(object instance, string methodName, params object[] param)
        {
            try
            {
                return instance.GetType().GetMethod(methodName, BindingFlags.Instance | BindingFlags.NonPublic).
                    Invoke(instance, param);
            }
            catch (TargetInvocationException e)
            {
                throw e.InnerException;
            }
        }

        /// <summary>
        /// Gets the field in the instance. The field can be private or protected.
        /// </summary>
        /// <param name="instance">The instance whose field is retrieved.</param>
        /// <param name="field">The field to be retrieved.</param>
        /// <returns>The value of the field in the instance.</returns>
        [ReflectionPermission(SecurityAction.PermitOnly, Unrestricted = true)]
        public static object GetField(object instance, string field)
        {
            return instance.GetType().GetField(field, BindingFlags.NonPublic | BindingFlags.Instance).
                GetValue(instance);
        }

        /// <summary>
        /// Asserts the two given instances are the same. If the instances are lists, they are compared as lists. If
        /// the instances are dictionaries, they are compared as dictionaries. If the instances are bread crumb nodes,
        /// they are compared using their titles and URLs. Otherwise, it uses the assertion method in NUnit directly.
        /// </summary>
        /// <param name="expected">The expected value to be compared with.</param>
        /// <param name="actual">The actual value to be compared with.</param>
        /// <param name="message">The error message when the assertion fails.</param>
        /// <exception cref="AssertionException">The actual value is not the same as the expected value.</exception>
        public static void AreEqual(object expected, object actual, string message)
        {
            if ((expected is IList) && (actual is IList))
            {
                AreEqual((IList)expected, (IList)actual, message);
            }
            else if ((expected is IDictionary) && (actual is IDictionary))
            {
                AreEqual((IDictionary)expected, (IDictionary)actual, message);
            }
            else if ((expected is BreadCrumbNode) && (actual is BreadCrumbNode))
            {
                Assert.AreEqual(((BreadCrumbNode)expected).Title, ((BreadCrumbNode)actual).Title, message);
                Assert.AreEqual(((BreadCrumbNode)expected).Url, ((BreadCrumbNode)actual).Url, message);
            }
            else
            {
                Assert.AreEqual(expected, actual, message);
            }
        }

        /// <summary>
        /// Asserts the two given dictionary are the same. The key/value entries must be the same.
        /// </summary>
        /// <param name="expected">The expected dictionary to be compared with.</param>
        /// <param name="actual">The actual dictionary to be compared with.</param>
        /// <param name="message">The error message when the assertion fails.</param>
        /// <exception cref="AssertionException">The actual dictionary is not the same as the expected dictionary.
        /// </exception>
        public static void AreEqual(IDictionary expected, IDictionary actual, string message)
        {
            // Verify the counts of redirection
            Assert.AreEqual(expected.Count, actual.Count, message);

            // Verify every entry
            foreach (DictionaryEntry entry in expected)
            {
                AreEqual(entry.Value, actual[entry.Key], message);
            }
        }

        /// <summary>
        /// Asserts the two given lists are the same. The list items must be the same.
        /// </summary>
        /// <param name="expected">The expected list to be compared with.</param>
        /// <param name="actual">The actual list to be compared with.</param>
        /// <param name="message">The error message when the assertion fails.</param>
        /// <exception cref="AssertionException">The actual list is not the same as the expected list.</exception>
        public static void AreEqual(IList expected, IList actual, string message)
        {
            // Verify the counts of redirection
            Assert.AreEqual(expected.Count, actual.Count, message);

            // Verify every item
            for (int i = 0; i < expected.Count; ++i)
            {
                AreEqual(expected[i], actual[i], message);
            }
        }

        /// <summary>
        /// Creates an array of <c>NodePair</c> instances. This method is used to create a previous path in the session,
        /// since <c>NodePair</c> is an internal class.
        /// </summary>
        /// <param name="nodes">The array of nodes used to create the node pair array.</param>
        /// <param name="matchers">The array of node matchers used to create the node pair array.</param>
        /// <returns>An array of <c>NodePair</c> instances.</returns>
        public static Array CreateNodePairArray(BreadCrumbNode[] nodes, INodeMatcher[] matchers)
        {
            NodePair[] pairs = new NodePair[nodes.Length];

            // Create the node pair using each node and matcher
            for (int index = 0; index < nodes.Length; ++index)
            {
                pairs[index] = new NodePair(nodes[index], matchers[index]);
            }

            return pairs;
        }
    }
}

